/*
  LCD - Library for most LCD charater displays, has common features 
  Copyright (c) 2013 Jim Spence.  All right reserved.
  www.byvac.com - see terms and conditions for using hardware
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
  Release Notes:
  bv4619 - LCD controller with keypad interface that can so both serial and I2C

*/  

#include "Arduino.h"
#include "bv4619_S.h"

// =============================================================================
// LCD Dsiplay section
// =============================================================================
// =============================================================================
// *****************************************************************************
// constructor is just a connection to NewSoftSerial
// *****************************************************************************
LCD::LCD(uint8_t rxPin, uint8_t txPin, long rbaud, char address) : BSerial(rxPin,txPin)
{
    _ack = 6;
    _adr = address;
    baud(rbaud);
}

// *****************************************************************************
// Clear screen on most displays is cmd 1
// *****************************************************************************
void LCD::lcd_cls()
{
    putch(_adr);
    puts("c1\r");
    delay(300);
}

// *****************************************************************************
// select a row and column
// row and column start at 0
// *****************************************************************************
void LCD::lcd_rowcol(char row, char col)
{
const int lcdrows[]={0x80,0xc0,0x94,0xd4}; 
int rc;
char buf[15];
    if(row < 0) row = 0;
    if(row > 3) row = 3;
    rc = lcdrows[row] + col;
    putch(_adr);
    sprintf(buf, "c%d\r", rc);
    puts(buf);  
    wack();  
}

// *****************************************************************************
// cursor control
// 0 turns cursot off
// 1 
// *****************************************************************************
void LCD::lcd_cursor(char cv)
{
    switch(cv) {
        case 0: { // turn off cursor
            putch(_adr);
            puts("c12\r");
            break;
        }
        case 1: { // cursor on normal
            putch(_adr);
            puts("c14\r");
            break;
        }
        case 2: { // blok flashing
            putch(_adr);
            puts("c13\r");
            break;
        }
    }
    wack();
}

// *****************************************************************************
// Print overloaded 
// *****************************************************************************
// String
void LCD::lcd_print(char *value)
{
    putch(_adr);
    putch('d');    
    puts(value);
    putch(13);    
    delay(1); wack();
}
// Int
void LCD::lcd_print(int value)
{
char buf[15];
    sprintf(buf, "d%d", value);
    putch(_adr);
    puts(buf);
    putch(13);
    delay(1); wack();    
}

// *****************************************************************************
// back light control
// RGB has 3 back lights
// *****************************************************************************
void LCD::lcd_bl(char red, char green, char blue)
{
char buf[15];
    if(red > 10) red = 10;
    if(green > 10 ) green = 10;
    if(blue > 10) blue =10;
    putch(_adr);
    sprintf(buf, "b%d,%d,%d\r",red,green,blue);
    puts(buf);
    wack();
}

// *****************************************************************************
// define custom character
// *****************************************************************************
void LCD::lcd_defcust(char position, char *definition)
{
char adr = 0x40;
char buf[15], *bp=definition;
    if(position > 7) position = 7;
    adr += 8 * position;
    // set address using command c
    putch(_adr);
    sprintf(buf, "c%d\r", adr);
    puts(buf);
    wack();//delay(100);
    // now write definition
    putch(_adr);
    putch('d');
    putch(*(bp++));
    putch(*(bp++));
    putch(*(bp++));
    putch(*(bp++));
    putch(*(bp++));
    putch(*(bp++));
    putch(*(bp++));
    putch(*(bp++));
    putch(13);
    wack();//delay(100);
}

// *****************************************************************************
// prints a custom char
// *****************************************************************************
void LCD::lcd_printcust(char address)
{
char buf[15];
    if(address > 7) address = 7;
    putch(_adr);
    putch('d');
    putch(address);
    putch(13);
    wack();
}    

// =============================================================================
// LCD Keypad section
// =============================================================================
// =============================================================================

// *****************************************************************************
// Gets a character from the input buffer with a timeout, if there is no char
// it will return -1
// as set if no ack then delays about 300mS
// *****************************************************************************
char LCD::getch_t()
{
unsigned int j=10000; // timeout
char c; 

    c = 0; // default
    while(j-- > 0) {
        c = getch();
        if(c != -1) break;
        delay(1);
    }
    return c;
}            

// *****************************************************************************
// for commands that just return ACK,
// Does not return anything but will wait for the ACK char or timeout
// *****************************************************************************
void LCD::wack()
{
char k=0;
    while(k != _ack) {
        k = getch_t();
        if(k == -1) break; // timed out
    }
}

// *****************************************************************************
// reads characters up to ACK and returns them, supplu buffer for return
// string, returns with number of charatcers in buffer
// *****************************************************************************
int LCD::read(char *buf)
{
char k,v=0;

    while(k != _ack) {
        k = getch_t();
        if((k == -1) || (k == _ack)) break; // timout or ack
        *(buf++)=k;
        *buf=0; // terminate each time        
        v++;        
    }
    return v;
}

// *****************************************************************************
// returns a key scan code, if no keys are in buffer returns 0
// *****************************************************************************
int LCD::lcd_getkey()
{
char buf[15];
    flush(); // clear buffer
    putch(_adr);
    putch('g');
    putch(13);
    read(buf);
    return atoi(buf);
}

// *****************************************************************************
// returns the number of keys in the buffer
// *****************************************************************************
int LCD::lcd_keybuf()
{
char buf[15];
    flush(); // clear buffer
    putch(_adr);
    putch('n');
    putch(13);
    read(buf);
    return atoi(buf);
}

// *****************************************************************************
// clears ceypad buffer
// *****************************************************************************
void LCD::lcd_keyclear()
{
char buf[15];
    putch(_adr);
    putch('r');
    putch(13);
    wack();
}
